"""
byte_elastalert_utils.elasticsearch.query
-------------------------------------------
functionality to query ElasticSearch Server
"""
from elasticsearch import Elasticsearch
import byte_elastalert_utils.elasticsearch.settings as settings


QUERY_RESULT_SIZE = 10000
MINUTES_IN_24_HOURS = 1440
INDEX_PREFIXES = {
    'heartbeat': 'heartbeat-',
}


def get_latest_heartbeat_docs(hostname, minutes=15, size=QUERY_RESULT_SIZE):
    """Returns list of Heartbeat documents for the specified hostname from the
    last few minutes.

    Note: Can only search for documents in last 24 hours, which is fine because
    we're mostly interested in latest information.

    :param str hostname: the server hostname (monitor.host)
    :param int minutes: Number of minutes to search for data (can only be stretch for current day)
    :param int size: max size of results
    :return: list[dict]: list of documents (each as a dict) sorted decendingly by timestamp
    :raises ValueError: on minutes more than 1440
    """
    if minutes > MINUTES_IN_24_HOURS:
        raise ValueError("Can't search for Heartbeat docs of {} minutes ago, it's more than 24 hours".format(minutes))

    request_body = {
        "query": {
            "bool": {
                "must": {
                    "match": {"monitor.host": hostname}
                },
                "filter": {
                    "range": {"@timestamp": {"gte": "now-{}m".format(minutes)}}
                }
            }
        }
    }

    sort_by = ['@timestamp:desc']
    index_suffixes = ['{now/d}', '{now/d-1d}']  # search for latest heartbeat docs, only in today and yesterday indexes
    indexes = ['<{}{}>'.format(INDEX_PREFIXES['heartbeat'], suffix) for suffix in index_suffixes]
    response = search(request_body, indexes, sort_by, size)
    return response['hits']['hits']


def search(body, indices, sort_by=(), size=QUERY_RESULT_SIZE):
    """Run a query to ElasticSearch

    :param dict body: the search request body
    :param list indices: list of index names to query
    :param iterable sort_by: iterable of <field>:<direction> values
    :param int size: limit the size of returned documents
    :return dict: response as dict
    """
    # using the elasticserch-py library to query ES, instead of elasticsearch-dsl
    # because ElastAlert does not use (nor install) elasticsearch-dsl (yet)
    client = get_elasticsearch_client()
    return client.search(body=body, index=','.join(indices), size=size, sort=','.join(sort_by))


def get_elasticsearch_client():
    """Return an ElasticSearch client to run queries with
    :rtype: elasticsearch.Elasticsearch:
    """
    con_settings = {
        'host': settings.ES_HOST,
        'port': settings.ES_PORT,
        'http_auth': (settings.ES_USERNAME, settings.ES_PASSWORD),
        'use_ssl': settings.ES_USE_SSL,
        'verify_certs': settings.ES_VERIFY_CERTS,
    }
    return Elasticsearch(**con_settings)
