"""
elasticsearch.settings
----------------------
ElasticSearch connection settings used by other modules in this package
"""
import os
import json


SETTINGS_FILE_PATH = '/etc/elastalert/byte-elasticsearch.json'


def _bool_env(name, default):
    """Returns a bool if the environment variable is set to 'yes/YES'
    or the specified default bool

    these values are considered as True: YES/TRUE/1
    anything else is Flase.

    :param str name: the env name
    :param bool default: the value to return if env is not set
    :return bool:
    """
    default_val = 'YES' if default else 'NO'
    return os.environ.get(name, default_val).strip().upper() in ('YES', 'TRUE', '1')


def read_json_conf(filepath):
    """Read settings from a JSON config file and return a dict"""

    with open(filepath, 'rt') as fh:
        contents = fh.read()
    conf = json.loads(contents)
    if not isinstance(conf, dict):
        raise RuntimeError(
                "Invalid settings in file '{}'. JSON file should contain an object".format(filepath)
              )
    return conf


conf = read_json_conf(SETTINGS_FILE_PATH) if os.path.exists(SETTINGS_FILE_PATH) else {}

# settings are read from a file (if exists), and are overridden by env vars
# use the same env names that ElastAlert uses
# and keep settings same as env names
ES_HOST = os.environ.get('ES_HOST', conf.get('ES_HOST', ''))
ES_PORT = int(os.environ.get('ES_PORT', conf.get('ES_PORT', 9200)))
ES_USERNAME = os.environ.get('ES_USERNAME', conf.get('ES_USERNAME', ''))
ES_PASSWORD = os.environ.get('ES_PASSWORD', conf.get('ES_PASSWORD', ''))
ES_USE_SSL = _bool_env('ES_USE_SSL', conf.get('ES_USE_SSL', True))
ES_VERIFY_CERTS = _bool_env('ES_VERIFY_CERTS', conf.get('ES_VERIFY_CERTS', False))
