"""
Byte ElastAlert Enhancements
"""
from elastalert.enhancements import BaseEnhancement, DropMatchException
from byte_elastalert_utils.common.log import get_logger
from byte_elastalert_utils.elasticsearch.query import get_latest_heartbeat_docs
from byte_elastalert_utils.hypernode.common import get_app_name
from byte_elastalert_utils.hypernode.hnapi import get_api_token, HnApi


MONITOR_STATUS_UP = 'up'

logger = get_logger(__name__)


def get_hypernode_info(app_host):
    """Return the info from Hypernode API of the given app/hostname
    :param str app_host: app name or hostname
    """
    app_name = get_app_name(app_host)
    hnapi_token = get_api_token()
    hnapi_client = HnApi(hnapi_token)
    return hnapi_client.get_app_info(app_name)


class PassStatusUpEnhancement(BaseEnhancement):
    """This enhancement ensures we only continue with the alert
    when the status is "up".  All other statuses are dropped.
    """

    def process(self, match):
        status = match['monitor']['status']
        if str(status).lower() != MONITOR_STATUS_UP:
            logger.info('Dropping match for "{}". monitor.status is not "up"'.format(match['monitor']['host']))
            raise DropMatchException()


class PassMinDownEventsEnhancement(BaseEnhancement):
    """This enhancement ensures we only continue with the alert
    when a minimum percentage of events were "down" (technically not "up").
    The threshold can be set on the class attribute 'min_down_events',

    This can be used to ensure the alert won't trigger of momentary down times.
    """
    timeframe_minutes = 15
    min_down_events_percent = 50

    def process(self, match):
        hostname = match['monitor']['host']
        heartbeat_docs = get_latest_heartbeat_docs(hostname, self.timeframe_minutes)
        if not heartbeat_docs:
            logger.warning('Querying ElasticSearch for Heartbeat docs for "{}" did not return resutls!'.format(hostname))
            return
        total_docs = len(heartbeat_docs)
        logger.info('Received {} HeartBeat docs for "{}"'.format(total_docs, hostname))
        down_events = [doc for doc in heartbeat_docs if doc['_source']['monitor']['status'] != MONITOR_STATUS_UP]
        down_events_percentage = 100 * (float(len(down_events)) / total_docs)  # example: 100 * (7.0 / 10) => 100 * 0.7 => 70%
        if down_events_percentage < self.min_down_events_percent:
            logger.info(
                'Dropping match for "{}". less than {}% of events are "down/-" events'.format(
                    hostname,
                    self.min_down_events_percent
                )
            )
            raise DropMatchException()


class AlertIfEnabledEnhancement(BaseEnhancement):
    def process(self, match):
        should_drop = False
        hostname = match['monitor']['host']
        logger.info('Checking if alerting is disabled for {}'.format(hostname))
        try:
            app_info = get_hypernode_info(hostname)
            if app_info.get('alerting_enabled') is False:  # don't drop unless explicitly disabled
                should_drop = True
        except Exception as err:
            logger.exception('Error checking for alerting status for "{}" from Hypernode API. {}'.format(hostname, err))
        if should_drop:
            logger.info('Alerting is disabled for {}. Dropping the alert'.format(hostname))
            raise DropMatchException()
